Package: psychotools
Title: Psychometric Modeling Infrastructure
Version: 0.7-7
Date: 2026-07-11
Authors@R: c(person(given = "Achim", family = "Zeileis", role = c("aut", "cre"), email = "Achim.Zeileis@R-project.org", comment = c(ORCID = "0000-0003-0918-3766")),
             person(given = "Carolin", family = "Strobl", role = "aut", email = "Carolin.Strobl@psychologie.uzh.ch", comment = c(ORCID = "0000-0003-0952-3230")),
             person(given = "Florian", family = "Wickelmaier", role = "aut"),
             person(given = "Basil", family = "Komboz", role = "aut"),
             person(given = "Julia", family = "Kopf", role = "aut"),
             person(given = "Lennart", family = "Schneider", role = "aut", comment = c(ORCID = "0000-0003-4152-5308")),
             person(given = "Rudolf", family = "Debelak", role = "aut", comment = c(ORCID = "0000-0001-8900-2106")))
Depends: R (>= 3.6.0)
Imports: graphics, grDevices, stats, utils
Suggests: car, Formula, likert, lmtest, mirt, multcomp, psychotree,
        psychomix, sandwich, strucchange
Description: Infrastructure for psychometric modeling such as data classes (for
  item response data and paired comparisons), basic model fitting functions (for
  Bradley-Terry, Rasch, parametric logistic IRT, generalized partial credit,
  rating scale, multinomial processing tree models), extractor functions for
  different types of parameters (item, person, threshold, discrimination,
  guessing, upper asymptotes), unified inference and visualizations, and various
  datasets for illustration.  Intended as a common lightweight and efficient
  toolbox for psychometric modeling and a common building block for fitting
  psychometric mixture models in package "psychomix" and trees based on
  psychometric models in package "psychotree".
License: GPL-2 | GPL-3
NeedsCompilation: yes
Packaged: 2026-07-11 00:10:59 UTC; zeileis
Author: Achim Zeileis [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0918-3766>),
  Carolin Strobl [aut] (ORCID: <https://orcid.org/0000-0003-0952-3230>),
  Florian Wickelmaier [aut],
  Basil Komboz [aut],
  Julia Kopf [aut],
  Lennart Schneider [aut] (ORCID:
    <https://orcid.org/0000-0003-4152-5308>),
  Rudolf Debelak [aut] (ORCID: <https://orcid.org/0000-0001-8900-2106>)
Maintainer: Achim Zeileis <Achim.Zeileis@R-project.org>
Repository: CRAN
Date/Publication: 2026-07-11 02:30:02 UTC
